@@ -64,30 +64,40 @@ find_config <- function(filename) {
 #'
 #' Lint and diagnose problems in a file.
 #' @keywords internal
-diagnose_file <- function(path, content = NULL) {
+diagnose_file <- function(uri, content) {
+    path <- path_from_uri(uri)
     if (is.null(find_config(path))) {
         linters <- getOption("languageserver.default_linters", NULL)
     } else {
         linters <- NULL
     }
-    if (is.null(content)) {
+
+    if (length(content) <= 1 && file.exists(path)) {
+        # lintr::lint doesn't work with one line text
         content <- readr::read_lines(path)
         diagnostics <- lapply(
             lintr::lint(path, linters = linters), diagnostic_from_lint, content = content)
     } else {
+        if (is_rmarkdown(path)) {
+            # make sure Rmarkdown file has at least one block
+            if (!any(stringr::str_detect(content, "```\\{r[ ,\\}]"))) {
+                return(list())
+            }
+        }
         # use inline data
         text <- paste0(content, collapse = "\n")
         diagnostics <- lapply(
             lintr::lint(text, linters = linters), diagnostic_from_lint, content = content)
+
     }
     names(diagnostics) <- NULL
     diagnostics
 }
 
 
-diagnostics_callback <- function(self, uri, diagnostics) {
+diagnostics_callback <- function(self, uri, version, diagnostics) {
     if (is.null(diagnostics)) return(NULL)
-    logger$info("diagnostics_callback called")
+    logger$info("diagnostics_callback called:", list(uri = uri, version = version))
     self$deliver(
         Notification$new(
             method = "textDocument/publishDiagnostics",
@@ -101,13 +111,11 @@ diagnostics_callback <- function(self, uri, diagnostics) {
 
 
 diagnostics_task <- function(self, uri, document) {
-    if (is.null(document)) {
-        content <- NULL
-    } else {
-        content <- document$content
-    }
+    version <- document$version
+    content <- document$content
     create_task(
-        diagnose_file,
-        list(path = path_from_uri(uri), content = content),
-        callback = function(result) diagnostics_callback(self, uri, result))
+        package_call(diagnose_file),
+        list(uri = uri, content = content),
+        callback = function(result) diagnostics_callback(self, uri, version, result),
+        error = function(e) logger$info("diagnostics_task:", e))
 }
