@@ -10,6 +10,9 @@ on_initialize <- function(self, id, params) {
     self$rootPath <- path_from_uri(self$rootUri)
     self$initializationOptions <- params$initializationOptions
     self$ClientCapabilities <- params$capabilities
+    ServerCapabilities <- merge_list(
+        ServerCapabilities,
+        getOption("languageserver.server_capabilities"))
     self$deliver(Response$new(id = id, result = list(capabilities = ServerCapabilities)))
 }
 
@@ -20,15 +23,17 @@ on_initialize <- function(self, id, params) {
 #' @keywords internal
 on_initialized <- function(self, params) {
     logger$info("on_initialized")
-    if (is_package(self$rootUri)) {
-
-        project_root <- path_from_uri(self$rootUri)
+    project_root <- self$rootPath
+    if (is_package(project_root)) {
         source_dir <- file.path(project_root, "R")
         files <- list.files(source_dir)
         for (f in files) {
             logger$info("load ", f)
-            uri <- path_to_uri(file.path(source_dir, f))
-            self$text_sync(uri, document = NULL, parse = TRUE)
+            path <- file.path(source_dir, f)
+            uri <- path_to_uri(path)
+            doc <- Document$new(uri, NULL, readr::read_lines(path))
+            self$workspace$documents$set(uri, doc)
+            self$text_sync(uri, document = doc, parse = TRUE)
         }
         deps <- tryCatch(desc::desc_get_deps(project_root), error = function(e) NULL)
         if (!is.null(deps)) {
