@@ -5,7 +5,7 @@
 text_document_completion  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
     self$deliver(completion_reply(id, uri, self$workspace, document, point))
 }
@@ -27,7 +27,7 @@ completion_item_resolve  <- function(self, id, params) {
 text_document_hover  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
     self$deliver(hover_reply(id, uri, self$workspace, document, point))
 }
@@ -40,7 +40,7 @@ text_document_hover  <- function(self, id, params) {
 text_document_signature_help  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
     self$deliver(signature_reply(id, uri, self$workspace, document, point))
 }
@@ -52,7 +52,7 @@ text_document_signature_help  <- function(self, id, params) {
 text_document_definition  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
     self$deliver(definition_reply(id, uri, self$workspace, document, point))
 }
@@ -89,7 +89,7 @@ text_document_references  <- function(self, id, params) {
 text_document_document_highlight  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     point <- document$from_lsp_position(params$position)
     self$deliver(document_highlight_reply(id, uri, self$workspace, document, point))
 }
@@ -101,7 +101,19 @@ text_document_document_highlight  <- function(self, id, params) {
 text_document_document_symbol  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    self$deliver(document_symbol_reply(id, uri, self$workspace))
+    document <- self$workspace$documents$get(uri)
+    reply <- document_symbol_reply(id, uri, self$workspace, document,
+        self$ClientCapabilities$textDocument$documentSymbol)
+    if (is.null(reply)) {
+        queue <- self$pending_replies$get(uri)[["textDocument/documentSymbol"]]
+        queue$push(list(
+            id = id,
+            version = document$version,
+            params = params
+        ))
+    } else {
+        self$deliver(reply)
+    }
 }
 
 #' `textDocument/codeAction` request handler
@@ -134,7 +146,20 @@ code_lens_resolve  <- function(self, id, params) {
 #' Handler to the `textDocument/documentLink` [Request].
 #' @keywords internal
 text_document_document_link  <- function(self, id, params) {
-
+    textDocument <- params$textDocument
+    uri <- textDocument$uri
+    document <- self$workspace$documents$get(uri)
+    reply <- document_link_reply(id, uri, self$workspace, document, self$rootPath)
+    if (is.null(reply)) {
+        queue <- self$pending_replies$get(uri)[["textDocument/documentLink"]]
+        queue$push(list(
+            id = id,
+            version = document$version,
+            params = params
+        ))
+    } else {
+        self$deliver(reply)
+    }
 }
 
 #' `documentLink/resolve` request handler
@@ -150,7 +175,20 @@ document_link_resolve  <- function(self, id, params) {
 #' Handler to the `textDocument/documentColor` [Request].
 #' @keywords internal
 text_document_document_color  <- function(self, id, params) {
-
+    textDocument <- params$textDocument
+    uri <- textDocument$uri
+    document <- self$workspace$documents$get(uri)
+    reply <- document_color_reply(id, uri, self$workspace, document)
+    if (is.null(reply)) {
+        queue <- self$pending_replies$get(uri)[["textDocument/documentColor"]]
+        queue$push(list(
+            id = id,
+            version = document$version,
+            params = params
+        ))
+    } else {
+        self$deliver(reply)
+    }
 }
 
 #' `textDocument/colorPresentation` request handler
@@ -158,7 +196,11 @@ text_document_document_color  <- function(self, id, params) {
 #' Handler to the `textDocument/colorPresentation` [Request].
 #' @keywords internal
 text_document_color_presentation  <- function(self, id, params) {
-
+    textDocument <- params$textDocument
+    uri <- textDocument$uri
+    document <- self$workspace$documents$get(uri)
+    color <- params$color
+    self$deliver(color_presentation_reply(id, uri, self$workspace, document, color))
 }
 
 #' `textDocument/formatting` request handler
@@ -169,7 +211,7 @@ text_document_formatting  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
     options <- params$options
-    self$deliver(formatting_reply(id, uri, self$documents[[uri]], options))
+    self$deliver(formatting_reply(id, uri, self$workspace$documents$get(uri), options))
 }
 
 #' `textDocument/rangeFormatting` request handler
@@ -179,7 +221,7 @@ text_document_formatting  <- function(self, id, params) {
 text_document_range_formatting  <- function(self, id, params) {
     textDocument <- params$textDocument
     uri <- textDocument$uri
-    document <- self$documents[[uri]]
+    document <- self$workspace$documents$get(uri)
     range <- list(
         start = document$from_lsp_position(params$range$start),
         end = document$from_lsp_position(params$range$end)
@@ -194,7 +236,13 @@ text_document_range_formatting  <- function(self, id, params) {
 #' Handler to the `textDocument/onTypeFormatting` [Request].
 #' @keywords internal
 text_document_on_type_formatting  <- function(self, id, params) {
-
+    textDocument <- params$textDocument
+    uri <- textDocument$uri
+    document <- self$workspace$documents$get(uri)
+    point <- document$from_lsp_position(params$position)
+    ch <- params$ch
+    options <- params$options
+    self$deliver(on_type_formatting_reply(id, uri, document, point, ch, options))
 }
 
 
@@ -221,3 +269,11 @@ text_document_prepare_rename  <- function(self, id, params) {
 text_document_folding_range  <- function(self, id, params) {
 
 }
+
+#' `textDocument/selectionRange` request handler
+#'
+#' Handler to the `textDocument/selectionRange` [Request].
+#' @keywords internal
+text_document_selection_range <- function(self, id, params) {
+
+}
