@@ -8,9 +8,18 @@ suppressPackageStartupMessages({
 # a hack to make withr::defer_parent to work, see https://github.com/r-lib/withr/issues/123
 defer <- withr::defer
 
-language_client <- function(working_dir = getwd(), debug = FALSE, diagnostics = FALSE) {
+language_client <- function(working_dir = getwd(), diagnostics = FALSE, capabilities = NULL) {
+
+    if (nzchar(Sys.getenv("R_LANGSVR_LOG"))) {
+        script <- sprintf(
+            "languageserver::run(debug = '%s')",
+            normalizePath(Sys.getenv("R_LANGSVR_LOG"), "/", mustWork = FALSE))
+    } else {
+        script <- "languageserver::run()"
+    }
+
     client <- LanguageClient$new(
-        file.path(R.home("bin"), "R"), c("--slave", "-e", "languageserver::run()"))
+        file.path(R.home("bin"), "R"), c("--slave", "-e", script))
 
     client$notification_handlers <- list(
         `textDocument/publishDiagnostics` = function(self, params) {
@@ -20,7 +29,7 @@ language_client <- function(working_dir = getwd(), debug = FALSE, diagnostics =
         }
     )
 
-    client$start(working_dir = working_dir)
+    client$start(working_dir = working_dir, capabilities = capabilities)
     client$catch_callback_error <- FALSE
     # initialize request
     data <- client$fetch(blocking = TRUE)
@@ -67,15 +76,25 @@ did_open <- function(client, path) {
 
 
 did_save <- function(client, path) {
+    includeText <- tryCatch(
+        client$ServerCapabilities$textDocumentSync$save$includeText,
+        error = function(e) FALSE
+    )
+    if (includeText) {
+        text <- paste0(readr::read_lines(path), collapse = "\n")
+        params <- list(textDocument = list(uri = path_to_uri(path)), text = text)
+    } else {
+        params <- list(textDocument = list(uri = path_to_uri(path)))
+    }
     notify(
         client,
         "textDocument/didSave",
-        list(textDocument = list(uri = path_to_uri(path))))
+        params)
     invisible(client)
 }
 
 
-respond <- function(client, method, params, timeout, retry=TRUE,
+respond <- function(client, method, params, timeout, retry = TRUE,
                             retry_when = function(result) length(result) == 0) {
     if (missing(timeout)) {
         if (Sys.getenv("R_COVR", "") == "true") {
@@ -122,7 +141,8 @@ respond_completion <- function(client, path, pos, ...) {
         "textDocument/completion",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            position = list(line = pos[1], character = pos[2])),
+            position = list(line = pos[1], character = pos[2])
+        ),
         ...
     )
 }
@@ -142,7 +162,8 @@ respond_signature <- function(client, path, pos, ...) {
         "textDocument/signatureHelp",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            position = list(line = pos[1], character = pos[2])),
+            position = list(line = pos[1], character = pos[2])
+        ),
         ...
     )
 }
@@ -153,7 +174,8 @@ respond_hover <- function(client, path, pos, ...) {
         "textDocument/hover",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            position = list(line = pos[1], character = pos[2])),
+            position = list(line = pos[1], character = pos[2])
+        ),
         ...
     )
 }
@@ -164,7 +186,8 @@ respond_definition <- function(client, path, pos, ...) {
         "textDocument/definition",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            position = list(line = pos[1], character = pos[2])),
+            position = list(line = pos[1], character = pos[2])
+        ),
         ...
     )
 }
@@ -176,7 +199,8 @@ respond_formatting <- function(client, path, ...) {
         "textDocument/formatting",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            options = list(tabSize = 4, insertSpaces = TRUE)),
+            options = list(tabSize = 4, insertSpaces = TRUE)
+        ),
         ...
     )
 }
@@ -187,12 +211,107 @@ respond_range_formatting <- function(client, path, start_pos, end_pos, ...) {
         "textDocument/rangeFormatting",
         list(
             textDocument = list(uri = path_to_uri(path)),
-            range = range(position(start_pos[1], start_pos[2]), position(end_pos[1], end_pos[2])),
-            options = list(tabSize = 4, insertSpaces = TRUE)),
+            range = range(
+                start = position(start_pos[1], start_pos[2]),
+                end = position(end_pos[1], end_pos[2])
+            ),
+            options = list(tabSize = 4, insertSpaces = TRUE)
+        ),
         ...
     )
 }
 
+respond_folding_range <- function(client, path, ...) {
+    respond(
+        client,
+        "textDocument/foldingRange",
+        list(
+            textDocument = list(uri = path_to_uri(path))),
+        ...
+    )
+}
+
+respond_selection_range <- function(client, path, positions, ...) {
+    respond(
+        client,
+        "textDocument/selectionRange",
+        list(
+            textDocument = list(uri = path_to_uri(path)),
+            positions),
+        ...
+    )
+}
+
+respond_on_type_formatting <- function(client, path, pos, ch, ...) {
+    respond(
+        client,
+        "textDocument/onTypeFormatting",
+        list(
+            textDocument = list(uri = path_to_uri(path)),
+            position = position(pos[1], pos[2]),
+            ch = ch,
+            options = list(tabSize = 4, insertSpaces = TRUE)
+        ),
+        ...
+    )
+}
+
+
+respond_document_highlight <- function(client, path, pos, ...) {
+    respond(
+        client,
+        "textDocument/documentHighlight",
+        list(
+            textDocument = list(uri = path_to_uri(path)),
+            position = list(line = pos[1], character = pos[2])
+        ),
+        ...
+    )
+}
+
+respond_document_symbol <- function(client, path, ...) {
+    respond(
+        client,
+        "textDocument/documentSymbol",
+        list(
+            textDocument = list(uri = path_to_uri(path))
+        ),
+        ...
+    )
+}
+
+respond_workspace_symbol <- function(client, query, ...) {
+    respond(
+        client,
+        "workspace/symbol",
+        list(
+            query = query
+        ),
+        ...
+    )
+}
+
+respond_document_link <- function(client, path, ...) {
+    respond(
+        client,
+        "textDocument/documentLink",
+        list(
+            textDocument = list(uri = path_to_uri(path))
+        ),
+        ...
+    )
+}
+
+respond_document_color <- function(client, path, ...) {
+    respond(
+        client,
+        "textDocument/documentColor",
+        list(
+            textDocument = list(uri = path_to_uri(path))
+        ),
+        ...
+    )
+}
 
 wait_for <- function(client, method, timeout = 5) {
     storage <- new.env(parent = .GlobalEnv)
