@@ -11,7 +11,9 @@ test_that("Simple completion works", {
             "file.c",
             "fs::path",
             "foo$sol",
-            ".Mac"
+            ".Mac",
+            "grDev",
+            "TRU"
         ),
         temp_file)
 
@@ -33,7 +35,13 @@ test_that("Simple completion works", {
     expect_length(result$items, 0)
 
     result <- client %>% respond_completion(temp_file, c(4, 4))
-    expect_length(result$items %>% keep(~.$label == ".Machine"), 1)
+    expect_length(result$items %>% keep(~ .$label == ".Machine"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(5, 5))
+    expect_length(result$items %>% keep(~ .$label == "grDevices"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(6, 3))
+    expect_length(result$items %>% keep(~ .$label == "TRUE"), 1)
 })
 
 test_that("Completion of function arguments works", {
@@ -76,6 +84,37 @@ test_that("Completion of user function works", {
 
 })
 
+test_that("Completion of symbols in scope works", {
+    skip_on_cran()
+    client <- language_client()
+
+    withr::local_tempfile(c("temp_file"), fileext = ".R")
+    writeLines(
+        c(
+            "xvar0 <- rnorm(10)",
+            "my_fun <- function(xvar1) {",
+            "    xvar2 <- 1",
+            "    for (xvar3 in 1:10) {",
+            "        xvar",
+            "    }",
+            "}"
+        ),
+        temp_file
+    )
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(
+        temp_file, c(3, 12),
+        retry_when = function(result) length(result) == 0 || length(result$items) == 0
+    )
+
+    expect_length(result$items, 4)
+    expect_length(result$items %>% keep(~ .$label == "xvar0"), 1)
+    expect_length(result$items %>% keep(~ .$label == "xvar1"), 1)
+    expect_length(result$items %>% keep(~ .$label == "xvar2"), 1)
+    expect_length(result$items %>% keep(~ .$label == "xvar3"), 1)
+})
 
 test_that("Completion inside a package works", {
     skip_on_cran()
@@ -153,3 +192,59 @@ test_that("Completion item resolve works", {
     expect_match(resolve_result$documentation$value,
         "`.Machine` is a variable holding information on the numerical characteristics of the machine \\*\\*R\\*\\* is running on")
 })
+
+test_that("Completion in Rmarkdown works", {
+    skip_on_cran()
+    client <- language_client()
+
+    withr::local_tempfile(c("temp_file"), fileext = ".Rmd")
+    writeLines(
+        c(
+            "Title",
+            "",
+            "```{r}",
+            "str",
+            "file.c",
+            "fs::path",
+            "foo$sol",
+            ".Mac",
+            "grDev",
+            "TRU",
+            "```",
+            "str"
+        ),
+        temp_file
+    )
+
+    client %>% did_save(temp_file)
+
+    result <- client %>% respond_completion(temp_file, c(3, 3))
+
+    expect_length(result$items %>% keep(~ .$label == "strsplit"), 1)
+    expect_length(result$items %>% keep(~ .$label == "strrep"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(4, 6))
+    expect_length(result$items %>% keep(~ .$label == "file.choose"), 1)
+    expect_length(result$items %>% keep(~ .$label == "file.create"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(5, 8))
+    expect_true("path_real" %in% (result$items %>% map_chr(~ .$label)))
+
+    result <- client %>% respond_completion(temp_file, c(6, 7))
+    expect_length(result$items, 0)
+
+    result <- client %>% respond_completion(temp_file, c(7, 4))
+    expect_length(result$items %>% keep(~ .$label == ".Machine"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(8, 5))
+    expect_length(result$items %>% keep(~ .$label == "grDevices"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(9, 3))
+    expect_length(result$items %>% keep(~ .$label == "TRUE"), 1)
+
+    result <- client %>% respond_completion(temp_file, c(10, 3))
+    expect_length(result$items, 0)
+
+    result <- client %>% respond_completion(temp_file, c(11, 3))
+    expect_length(result$items, 0)
+})
